@@ -155,7 +155,8 @@ server_sendquit (session * sess)
 
 void
 process_data_init (char *buf, char *cmd, char *word[],
-						 char *word_eol[], int handle_quotes)
+						 char *word_eol[], gboolean handle_quotes,
+						 gboolean allow_escape_quotes)
 {
 	int wordcount = 2;
 	int space = FALSE;
@@ -184,6 +185,12 @@ process_data_init (char *buf, char *cmd, char *word[],
 		case '\042':
 			if (!handle_quotes)
 				goto def;
+			/* two quotes turn into 1 */
+			if (allow_escape_quotes && cmd[1] == '\042')
+			{
+				cmd++;
+				goto def;
+			}
 			if (quote)
 			{
 				quote = FALSE;
@@ -396,7 +403,7 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 	{
 		if (deop)
 		{
-			mode = "-o+b";
+			mode = "-o+b ";
 			p2 = user->nick;
 		} else
 		{
@@ -458,19 +465,19 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 			switch (bantype)
 			{
 			case 0:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!*@%s.*", mode, p2, domain);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!*@%s.*", mode, p2, domain);
 				break;
 
 			case 1:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!*@%s", mode, p2, fullhost);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!*@%s", mode, p2, fullhost);
 				break;
 
 			case 2:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!%s@%s.*", mode, p2, username, domain);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!%s@%s.*", mode, p2, username, domain);
 				break;
 
 			case 3:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!%s@%s", mode, p2, username, fullhost);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!%s@%s", mode, p2, username, fullhost);
 				break;
 			}
 		} else
@@ -478,19 +485,19 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 			switch (bantype)
 			{
 			case 0:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!*@*%s", mode, p2, domain);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!*@*%s", mode, p2, domain);
 				break;
 
 			case 1:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!*@%s", mode, p2, fullhost);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!*@%s", mode, p2, fullhost);
 				break;
 
 			case 2:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!%s@*%s", mode, p2, username, domain);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!%s@*%s", mode, p2, username, domain);
 				break;
 
 			case 3:
-				snprintf (tbuf, TBUFSIZE, "%s %s *!%s@%s", mode, p2, username, fullhost);
+				snprintf (tbuf, TBUFSIZE, "%s%s *!%s@%s", mode, p2, username, fullhost);
 				break;
 			}
 		}
@@ -537,6 +544,45 @@ cmd_unban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 }
 
+static int
+cmd_chanopt (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int state;
+	if (!strcasecmp (word[3], "ON"))
+	{
+		state = TRUE;
+	} else if (!strcasecmp (word[3], "OFF"))
+	{
+		state = FALSE;
+	} else
+	{
+		if (!strcasecmp (word[2], "CONFMODE"))
+			state = sess->hide_join_part;
+		else if (!strcasecmp (word[2], "COLORPASTE"))
+			state = sess->color_paste;
+		else if (!strcasecmp (word[2], "BEEP"))
+			state = sess->beep;
+		else
+			return FALSE;
+
+		PrintTextf (sess, "%s is %s\n", word[2], state ? "ON" : "OFF");
+		return TRUE;
+	}
+
+	if (!strcasecmp (word[2], "CONFMODE"))
+	{
+		sess->hide_join_part = state;
+	} else if (!strcasecmp (word[2], "COLORPASTE"))
+	{
+		fe_set_color_paste (sess, state);
+	} else if (!strcasecmp (word[2], "BEEP"))
+	{
+		sess->beep = state;
+	}
+
+	return TRUE;
+}
+
 static int
 cmd_charset (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -574,6 +620,18 @@ cmd_clear (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	GSList *list = sess_list;
 	char *reason = word_eol[2];
 
+	if (reason[0] == 0)
+	{
+		fe_text_clear (sess);
+		return TRUE;
+	}
+
+	if (strcasecmp (reason, "HISTORY") == 0)
+	{
+		history_free (&sess->history);
+		return TRUE;
+	}
+
 	if (strncasecmp (reason, "all", 3) == 0)
 	{
 		while (list)
@@ -583,12 +641,10 @@ cmd_clear (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				fe_text_clear (list->data);
 			list = list->next;
 		}
-	} else
-	{
-		fe_text_clear (sess);
+		return TRUE;
 	}
 
-	return TRUE;
+	return FALSE;
 }
 
 static int
@@ -1088,7 +1144,7 @@ menu_del (char *path, char *label)
 }
 
 static void
-menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, int enable, int mod, int key)
+menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, int markup, int enable, int mod, int key)
 {
 	menu_entry *me;
 
@@ -1106,6 +1162,7 @@ menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, in
 	me = malloc (sizeof (menu_entry));
 	me->pos = pos;
 	me->state = state;
+	me->markup = markup;
 	me->enable = enable;
 	me->modifier = mod;
 	me->key = key;
@@ -1121,8 +1178,15 @@ menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, in
 	if (ucmd)
 		me->ucmd = strdup (ucmd);
 
-	fe_menu_add (me);
 	menu_list = g_slist_append (menu_list, me);
+	label = fe_menu_add (me);
+	if (label)
+	{
+		/* FE has given us a stripped label */
+		free (me->label);
+		me->label = strdup (label);
+		g_free (label); /* this is from pango */
+	}
 }
 
 static int
@@ -1134,6 +1198,7 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	int state;
 	int toggle = FALSE;
 	int enable = TRUE;
+	int markup = FALSE;
 	int key = 0;
 	int mod = 0;
 	char *label;
@@ -1159,10 +1224,17 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		idx++;
 	}
 
+	/* -m to specify PangoMarkup language */
+	if (word[idx][0] == '-' && word[idx][1] == 'm')
+	{
+		markup = TRUE;
+		idx++;
+	}
+
 	/* -pX to specify menu position */
 	if (word[idx][0] == '-' && word[idx][1] == 'p')
 	{
-		pos = atoi (word[2] + 2);
+		pos = atoi (word[idx] + 2);
 		idx++;
 	}
 
@@ -1185,17 +1257,25 @@ cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (label[0] == '-' && label[1] == 0)
 		label = NULL;	/* separator */
 
+	if (markup)
+	{
+		char *p;	/* to force pango closing tags through */
+		for (p = label; *p; p++)
+			if (*p == 3)
+				*p = '/';
+	}
+
 	if (!strcasecmp (word[idx], "ADD"))
 	{
 		if (toggle)
 		{
-			menu_add (tbuf, label, word[idx + 2], word[idx + 3], pos, state, enable, mod, key);
+			menu_add (tbuf, label, word[idx + 2], word[idx + 3], pos, state, markup, enable, mod, key);
 		} else
 		{
 			if (word[idx + 2][0])
-				menu_add (tbuf, label, word[idx + 2], NULL, pos, 0, enable, mod, key);
+				menu_add (tbuf, label, word[idx + 2], NULL, pos, 0, markup, enable, mod, key);
 			else
-				menu_add (tbuf, label, NULL, NULL, pos, 0, enable, mod, key);
+				menu_add (tbuf, label, NULL, NULL, pos, 0, markup, enable, mod, key);
 		}
 		return TRUE;
 	}
@@ -1808,6 +1888,57 @@ cmd_getint (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static void
+get_file_cb (char *cmd, char *file)
+{
+	char buf[1024 + 128];
+
+	/* execute the command once per file, then once more with
+      no args */
+	if (file)
+	{
+		snprintf (buf, sizeof (buf), "%s %s", cmd, file);
+		handle_command (current_sess, buf, FALSE);
+	}
+	else
+	{
+		handle_command (current_sess, cmd, FALSE);
+		free (cmd);
+	}
+}
+
+static int
+cmd_getfile (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int idx = 2;
+	int flags = 0;
+
+	if (!word[3][0])
+		return FALSE;
+
+	if (!strcmp (word[2], "-folder"))
+	{
+		flags |= FRF_CHOOSEFOLDER;
+		idx++;
+	}
+
+	if (!strcmp (word[idx], "-multi"))
+	{
+		flags |= FRF_MULTIPLE;
+		idx++;
+	}
+
+	if (!strcmp (word[idx], "-save"))
+	{
+		flags |= FRF_WRITE;
+		idx++;
+	}
+
+	fe_get_file (word[idx+1], word[idx+2], (void *)get_file_cb, strdup (word[idx]), flags);
+
+	return TRUE;
+}
+
 static void
 get_str_cb (int cancel, char *val, getvalinfo *info)
 {
@@ -1856,41 +1987,24 @@ cmd_ghost (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 static int
 cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	if (!strcasecmp (word[2], "HIDE"))
-	{
-		fe_ctrl_gui (sess, 0, 0);
-	} else if (!strcasecmp (word[2], "SHOW"))
-	{
-		fe_ctrl_gui (sess, 1, 0);
-	} else if (!strcasecmp (word[2], "FOCUS"))
-	{
-		fe_ctrl_gui (sess, 2, 0);
-	} else if (!strcasecmp (word[2], "FLASH"))
-	{
-		fe_ctrl_gui (sess, 3, 0);
-	} else if (!strcasecmp (word[2], "COLOR"))
-	{
-		fe_ctrl_gui (sess, 4, atoi (word[3]));
-	} else if (!strcasecmp (word[2], "ICONIFY"))
-	{
-		fe_ctrl_gui (sess, 5, 0);
-	} else if (!strcasecmp (word[2], "MENU"))
+	switch (str_ihash (word[2]))
 	{
+	case 0xac1eee45: fe_ctrl_gui (sess, 7, 2); break; /* ATTACH */
+	case 0x05a72f63: fe_ctrl_gui (sess, 4, atoi (word[3])); break; /* COLOR */
+	case 0xb06a1793: fe_ctrl_gui (sess, 7, 1); break; /* DETACH */
+	case 0x05cfeff0: fe_ctrl_gui (sess, 3, 0); break; /* FLASH */
+	case 0x05d154d8: fe_ctrl_gui (sess, 2, 0); break; /* FOCUS */
+	case 0x0030dd42: fe_ctrl_gui (sess, 0, 0); break; /* HIDE */
+	case 0x61addbe3: fe_ctrl_gui (sess, 5, 0); break; /* ICONIFY */
+	case 0xc0851aaa: fe_message (word[3], FE_MSG_INFO|FE_MSG_MARKUP); break; /* MSGBOX */
+	case 0x0035dafd: fe_ctrl_gui (sess, 1, 0); break; /* SHOW */
+	case 0x0033155f: /* MENU */
 		if (!strcasecmp (word[3], "TOGGLE"))
 			fe_ctrl_gui (sess, 6, 0);
 		else
 			return FALSE;
-	} else if (!strcasecmp (word[2], "MSGBOX"))
-	{
-		fe_message (word[3], FE_MSG_INFO);
-	} else if (!strcasecmp (word[2], "ATTACH"))
-	{
-		fe_ctrl_gui (sess, 7, 2);
-	} else if (!strcasecmp (word[2], "DETACH"))
-	{
-		fe_ctrl_gui (sess, 7, 1);
-	} else
-	{
+		break;
+	default:
 		return FALSE;
 	}
 
@@ -1910,6 +2024,9 @@ show_help_line (session *sess, help_list *hl, char *name, char *usage)
 	int j, len, max;
 	char *p;
 
+	if (name[0] == '.')	/* hidden command? */
+		return;
+
 	if (hl->longfmt)	/* long format for /HELP -l */
 	{
 		if (!usage || usage[0] == 0)
@@ -2190,7 +2307,7 @@ cmd_lagcheck (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 }
 
 static void
-lastlog (session *sess, char *search)
+lastlog (session *sess, char *search, gboolean regexp)
 {
 	session *lastlog_sess;
 
@@ -2200,11 +2317,12 @@ lastlog (session *sess, char *search)
 	lastlog_sess = find_dialog (sess->server, "(lastlog)");
 	if (!lastlog_sess)
 		lastlog_sess = new_ircwindow (sess->server, "(lastlog)", SESS_DIALOG, 0);
+
 	lastlog_sess->lastlog_sess = sess;
+	lastlog_sess->lastlog_regexp = regexp;	/* remember the search type */
 
 	fe_text_clear (lastlog_sess);
-
-	fe_lastlog (sess, lastlog_sess, search);
+	fe_lastlog (sess, lastlog_sess, search, regexp);
 }
 
 static int
@@ -2212,7 +2330,10 @@ cmd_lastlog (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	if (*word_eol[2])
 	{
-		lastlog (sess, word_eol[2]);
+		if (!strcmp (word[2], "-r"))
+			lastlog (sess, word_eol[3], TRUE);
+		else
+			lastlog (sess, word_eol[2], FALSE);
 		return TRUE;
 	}
 
@@ -2506,9 +2627,16 @@ static int
 cmd_notify (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int i = 1;
+	char *net = NULL;
 
 	if (*word[2])
 	{
+		if (strcmp (word[2], "-n") == 0)	/* comma sep network list */
+		{
+			net = word[3];
+			i += 2;
+		}
+
 		while (1)
 		{
 			i++;
@@ -2519,7 +2647,7 @@ cmd_notify (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				EMIT_SIGNAL (XP_TE_DELNOTIFY, sess, word[i], NULL, NULL, NULL, 0);
 				return TRUE;
 			}
-			notify_adduser (word[i]);
+			notify_adduser (word[i], net);
 			EMIT_SIGNAL (XP_TE_ADDNOTIFY, sess, word[i], NULL, NULL, NULL, 0);
 		}
 	} else
@@ -2578,25 +2706,32 @@ cmd_ping (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 }
 
 void
-open_query (server *serv, char *nick)
+open_query (server *serv, char *nick, gboolean focus_existing)
 {
 	session *sess;
 
 	sess = find_dialog (serv, nick);
-	if (sess)
-		fe_ctrl_gui (sess, 2, 0);	/* bring-to-front */
-	else
+	if (!sess)
 		new_ircwindow (serv, nick, SESS_DIALOG, 1);
+	else if (focus_existing)
+		fe_ctrl_gui (sess, 2, 0);	/* bring-to-front */
 }
 
 static int
 cmd_query (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	char *nick = word[2];
+	gboolean focus = TRUE;
+
+	if (strcmp (word[2], "-nofocus") == 0)
+	{
+		nick = word[3];
+		focus = FALSE;
+	}
 
 	if (*nick && !is_channel (sess->server, nick))
 	{
-		open_query (sess->server, nick);
+		open_query (sess->server, nick, focus);
 		return TRUE;
 	}
 	return FALSE;
@@ -2762,6 +2897,18 @@ cmd_settext (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_splay (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (*word[2])
+	{
+		sound_play (word[2], FALSE);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 parse_irc_url (char *url, char *server_name[], char *port[], char *channel[], int *use_ssl)
 {
@@ -2885,6 +3032,13 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		/* -1 for default port */
 		serv->connect (serv, server_name, -1, FALSE);
 	}
+
+	/* try to associate this connection with a listed network */
+	if (!serv->network)
+		/* search for this hostname in the entire server list */
+		serv->network = servlist_net_find_from_server (server_name);
+		/* may return NULL, but that's OK */
+
 	return TRUE;
 }
 
@@ -2970,6 +3124,23 @@ cmd_unload (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static server *
+find_server_from_hostname (char *hostname)
+{
+	GSList *list = serv_list;
+	server *serv;
+
+	while (list)
+	{
+		serv = list->data;
+		if (!strcasecmp (hostname, serv->hostname) && serv->connected)
+			return serv;
+		list = list->next;
+	}
+
+	return NULL;
+}
+
 static server *
 find_server_from_net (void *net)
 {
@@ -2987,6 +3158,16 @@ find_server_from_net (void *net)
 	return NULL;
 }
 
+static void
+url_join_only (server *serv, char *tbuf, char *channel)
+{
+	/* already connected, JOIN only. FIXME: support keys? */
+	tbuf[0] = '#';
+	/* tbuf is 4kb */
+	safe_strcpy ((tbuf + 1), channel, 256);
+	serv->p_join (serv, tbuf, "");
+}
+
 static int
 cmd_url (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -3016,11 +3197,18 @@ cmd_url (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				serv = find_server_from_net (net);
 				if (serv)
 				{
-					/* already connected, JOIN only. FIXME: support keys? */
-					tbuf[0] = '#';
-					/* tbuf is 4kb */
-					safe_strcpy ((tbuf + 1), channel, 256);
-					serv->p_join (serv, tbuf, "");
+					url_join_only (serv, tbuf, channel);
+					g_free (url);
+					return TRUE;
+				}
+			}
+			else
+			{
+				/* an un-listed connection */
+				serv = find_server_from_hostname (server_name);
+				if (serv)
+				{
+					url_join_only (serv, tbuf, channel);
 					g_free (url);
 					return TRUE;
 				}
@@ -3207,8 +3395,14 @@ const struct commands xc_cmds[] = {
 	{"AWAY", cmd_away, 1, 0, 1, N_("AWAY [<reason>], sets you away")},
 	{"BAN", cmd_ban, 1, 1, 1,
 	 N_("BAN <mask> [<bantype>], bans everyone matching the mask from the current channel. If they are already on the channel this doesn't kick them (needs chanop)")},
+	{"CHANOPT", cmd_chanopt, 0, 0, 1,
+	 N_("Set per channel options\n"
+	 "CHANOPT CONFMODE ON|OFF - Toggle conf mode/showing of join and part messages\n"
+	 "CHANOPT COLORPASTE ON|OFF - Toggle color paste\n"
+	 "CHANOPT BEEP ON|OFF - Toggle beep on message"
+	)},
 	{"CHARSET", cmd_charset, 0, 0, 1, 0},
-	{"CLEAR", cmd_clear, 0, 0, 1, N_("CLEAR, Clears the current text window")},
+	{"CLEAR", cmd_clear, 0, 0, 1, N_("CLEAR [ALL|HISTORY], Clears the current text window or command history")},
 	{"CLOSE", cmd_close, 0, 0, 1, N_("CLOSE, Closes the current window/tab")},
 
 	{"COUNTRY", cmd_country, 0, 0, 1,
@@ -3258,6 +3452,7 @@ const struct commands xc_cmds[] = {
 	{"GATE", cmd_gate, 0, 0, 1,
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
 	{"GETINT", cmd_getint, 0, 0, 1, "GETINT <default> <command> <prompt>"},
+	{"GETFILE", cmd_getfile, 0, 0, 1, "GETFILE [-folder] [-multi] [-save] <command> <title> [<initial>]"},
 	{"GETSTR", cmd_getstr, 0, 0, 1, "GETSTR <default> <command> <prompt>"},
 	{"GHOST", cmd_ghost, 1, 0, 1, N_("GHOST <nick> <password>, Kills a ghosted nickname")},
 	{"GUI", cmd_gui, 0, 0, 1, "GUI [ATTACH|DETACH|SHOW|HIDE|FOCUS|FLASH|ICONIFY|COLOR <n>]\n"
@@ -3294,7 +3489,7 @@ const struct commands xc_cmds[] = {
 	 N_("MDEOP, Mass deop's all chanops in the current channel (needs chanop)")},
 	{"ME", cmd_me, 0, 0, 1,
 	 N_("ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)")},
-	{"MENU", cmd_menu, 0, 0, 1, "MENU [-eX] [-k<mod>,<key>] [-pX] [-tX] {ADD|DEL} <path> [command] [unselect command]"},
+	{"MENU", cmd_menu, 0, 0, 1, "MENU [-eX] [-k<mod>,<key>] [-m] [-pX] [-tX] {ADD|DEL} <path> [command] [unselect command]"},
 	{"MKICK", cmd_mkick, 1, 1, 1,
 	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
 	{"MODE", cmd_mode, 1, 0, 1, 0},
@@ -3312,7 +3507,7 @@ const struct commands xc_cmds[] = {
 	{"NOTICE", cmd_notice, 1, 0, 1,
 	 N_("NOTICE <nick/channel> <message>, sends a notice. Notices are a type of message that should be auto reacted to")},
 	{"NOTIFY", cmd_notify, 0, 0, 1,
-	 N_("NOTIFY [<nick>], lists your notify list or adds someone to it")},
+	 N_("NOTIFY [-n network1[,network2,...]] [<nick>], displays your notify list or adds someone to it")},
 	{"OP", cmd_op, 1, 1, 1,
 	 N_("OP <nick>, gives chanop status to the nick (needs chanop)")},
 	{"PART", cmd_part, 1, 1, 1,
@@ -3320,7 +3515,7 @@ const struct commands xc_cmds[] = {
 	{"PING", cmd_ping, 1, 0, 1,
 	 N_("PING <nick | channel>, CTCP pings nick or channel")},
 	{"QUERY", cmd_query, 0, 0, 1,
-	 N_("QUERY <nick>, opens up a new privmsg window to someone")},
+	 N_("QUERY [-nofocus] <nick>, opens up a new privmsg window to someone")},
 	{"QUIT", cmd_quit, 0, 0, 1,
 	 N_("QUIT [<reason>], disconnects from the current server")},
 	{"QUOTE", cmd_quote, 1, 0, 1,
@@ -3351,10 +3546,11 @@ const struct commands xc_cmds[] = {
 	{"SERVER", cmd_server, 0, 0, 1,
 	 N_("SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667")},
 #endif
-	{"SET", cmd_set, 0, 0, 1, N_("SET [-quiet] <variable> [<value>]")},
+	{"SET", cmd_set, 0, 0, 1, N_("SET [-e] [-quiet] <variable> [<value>]")},
 	{"SETCURSOR", cmd_setcursor, 0, 0, 1, N_("SETCURSOR [-|+]<position>")},
 	{"SETTAB", cmd_settab, 0, 0, 1, 0},
 	{"SETTEXT", cmd_settext, 0, 0, 1, 0},
+	{"SPLAY", cmd_splay, 0, 0, 1, "SPLAY <soundfile>"},
 	{"TOPIC", cmd_topic, 1, 1, 1,
 	 N_("TOPIC [<topic>], sets the topic if one is given, else shows the current topic")},
 	{"UNBAN", cmd_unban, 1, 1, 1,
@@ -3426,8 +3622,8 @@ help (session *sess, char *tbuf, char *helpcmd, int quiet)
 
 int
 auto_insert (char *dest, int destlen, unsigned char *src, char *word[],
-				 char *word_eol[], char *a, char *c, char *d, char *h, char *n,
-				 char *s)
+				 char *word_eol[], char *a, char *c, char *d, char *e, char *h,
+				 char *n, char *s)
 {
 	int num;
 	char buf[32];
@@ -3508,6 +3704,8 @@ auto_insert (char *dest, int destlen, unsigned char *src, char *word[],
 					utf = c; break;
 				case 'd':
 					utf = d; break;
+				case 'e':
+					utf = e; break;
 				case 'h':
 					utf = h; break;
 				case 'm':
@@ -3738,8 +3936,9 @@ static void
 user_command (session * sess, char *tbuf, char *cmd, char *word[],
 				  char *word_eol[])
 {
-	if (!auto_insert (tbuf, 2048, cmd, word, word_eol, "",
-							sess->channel, "", "", sess->server->nick, ""))
+	if (!auto_insert (tbuf, 2048, cmd, word, word_eol, "", sess->channel, "",
+							server_get_network (sess->server, TRUE), "",
+							sess->server->nick, ""))
 	{
 		PrintText (sess, _("Bad arguments for user command.\n"));
 		return;
@@ -3765,7 +3964,7 @@ handle_say (session *sess, char *text, int check_spch)
 
 	if (strcmp (sess->channel, "(lastlog)") == 0)
 	{
-		lastlog (sess->lastlog_sess, text);
+		lastlog (sess->lastlog_sess, text, sess->lastlog_regexp);
 		return;
 	}
 
@@ -3780,7 +3979,7 @@ handle_say (session *sess, char *text, int check_spch)
 		check_special_chars (text, prefs.perc_ascii);
 
 	/* split the text into words and word_eol */
-	process_data_init (pdibuf, text, word, word_eol, TRUE);
+	process_data_init (pdibuf, text, word, word_eol, TRUE, FALSE);
 
 	/* a command of "" can be hooked for non-commands */
 	if (plugin_emit_command (sess, "", word, word_eol))
@@ -3915,11 +4114,11 @@ handle_command (session *sess, char *cmd, int check_spch)
 		tbuf = tbuf_static;
 
 	/* split the text into words and word_eol */
-	process_data_init (pdibuf, cmd, word, word_eol, TRUE);
+	process_data_init (pdibuf, cmd, word, word_eol, TRUE, TRUE);
 	int_cmd = find_internal_command (word[1]);
 	/* redo it without quotes processing, for some commands like /JOIN */
 	if (int_cmd && !int_cmd->handle_quotes)
-		process_data_init (pdibuf, cmd, word, word_eol, FALSE);
+		process_data_init (pdibuf, cmd, word, word_eol, FALSE, FALSE);
 
 	if (check_spch && prefs.perc_color)
 		check_special_chars (cmd, prefs.perc_ascii);
