@@ -24,20 +24,18 @@
 #include "about.h"
 #include "../common/fe.h"
 
-static void about_email_hook        (GtkAboutDialog *dialog, const gchar *link, gpointer data);
-static void about_url_hook          (GtkAboutDialog *dialog, const gchar *link, gpointer data);
-
 void
 show_about_dialog (void)
 {
 	char *license_trans;
 
-	const gchar *authors[] =
-	{
+	const gchar *authors[] = {
 		"Andre Dahlqvist <andre.bugs@gmail.com>",
 		"Bastien Nocera <hadess@hadess.net>",
 		"Brian Pepple <bdpepple@gmail.com>",
+		"Christian Persch <chpe@gnome.org>",
 		"Dan Kuester <Daniel.Kuester@colorado.edu>",
+		"Daniel Drake <dsd@gentoo.org>"
 		"David Trowbridge <trowbrds@gmail.com>",
 		"Evan Sheehan <Wallace.Sheehan@gmail.com>",
 		"Guillaume Desmottes <cass@skynet.be>",
@@ -49,13 +47,14 @@ show_about_dialog (void)
 		"Xavier Claessens <xclaesse@gmail.com>",
 		NULL
 	};
-	const gchar *documenters[] =
-	{
+	const gchar *documenters[] = {
 		"Brian Pepple <bdpepple@gmail.com>",
 		NULL
 	};
-	const gchar *artists[] =
-	{
+	const gchar *artists[] = {
+		"Andreas Nilsson <nisses.mail@home.se>",
+		"Jones Lee <joneslee85@gmail.com>",
+		"Lapo Calamandrei <lapo@it.gnome.org>",
 		"Micah Dowty <micah@navi.cx>",
 		NULL
 	};
@@ -76,25 +75,22 @@ show_about_dialog (void)
 
 	license_trans = g_strjoin ("\n\n", _(license[0]), _(license[1]), _(license[2]), NULL);
 
-	gtk_about_dialog_set_url_hook   (about_url_hook,   NULL, NULL);
-	gtk_about_dialog_set_email_hook (about_email_hook, NULL, NULL);
-
 	gtk_show_about_dialog (GTK_WINDOW (gui.main_window),
-	                       "name",        "XChat-GNOME",
-	                       "version",     PACKAGE_VERSION,
-	                       "copyright",   _("Copyright © 2004-2006"),
-			       /* Translators: This is a random, mostly-meaningless
-				* quotation from Jeeves & Wooster.  Either leave it
-				* be or replace it with something funny (but inoffensive)
-				* in your own language, but don't translate it literally.
-				*/
-	                       "comments",    _("It has been well observed that a trombone is not a suitable instrument for a gentleman"),
-	                       "license",     license_trans,
+	                       "name", "XChat-GNOME",
+	                       "version", PACKAGE_VERSION,
+	                       "copyright", _("Copyright © 2004-2006"),
+	                       /* Translators: This is a random, mostly-meaningless
+	                        * quotation from Jeeves & Wooster.  Either leave it
+	                        * be or replace it with something funny (but inoffensive)
+	                        * in your own language, but don't translate it literally.
+	                        */
+	                       "comments", _("It has been well observed that a trombone is not a suitable instrument for a gentleman"),
+	                       "license", license_trans,
 	                       "wrap-license", TRUE,
-	                       "website",     "http://xchat-gnome.navi.cx",
+	                       "website", "http://xchat-gnome.navi.cx",
 	                       "website-label", _("XChat-GNOME Web Site"),
-	                       "authors",     authors,
-	                       "artists",     artists,
+	                       "authors", authors,
+	                       "artists", artists,
 	                       "logo-icon-name", "xchat-gnome",
 	                       "documenters", documenters,
 	                       /* Translators: This is a special message that shouldn't
@@ -111,14 +107,3 @@ show_about_dialog (void)
 
 	g_free (license_trans);
 }
-
-static void
-about_email_hook (GtkAboutDialog *dialog, const gchar *link, gpointer data)
-{
-}
-
-static void
-about_url_hook (GtkAboutDialog *dialog, const gchar *link, gpointer data)
-{
-	fe_open_url (link);
-}
