@@ -50,10 +50,11 @@ connect_dialog_class_init (ConnectDialogClass *klass)
 static void
 selection_changed (GtkTreeSelection *select, ConnectDialog *dialog)
 {
-	if (gtk_tree_selection_get_selected (select, NULL, NULL))
+	if (gtk_tree_selection_get_selected (select, NULL, NULL)) {
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, TRUE);
-	else
+	} else {
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, FALSE);
+	}
 }
 
 static void
@@ -71,12 +72,14 @@ dialog_response (ConnectDialog *dialog, gint response, gpointer data)
 			gboolean connected;
 
 			s = navigation_tree_get_selected_session(&connected);
-			if (connected)
+			if (connected) {
 				s = NULL;
+			}
 
 			gtk_tree_model_get (model, &iter, 0, &network, -1);
-			if (!navigation_tree_server_is_connected (gui.server_tree, network))
+			if (!navigation_tree_server_is_connected (gui.server_tree, network)) {
 				servlist_connect_by_netname (s, network, TRUE);
+			}
 			g_free (network);
 			gtk_widget_destroy (GTK_WIDGET (dialog));
 		}
@@ -89,8 +92,9 @@ static void
 row_activated (GtkTreeView *widget, GtkTreePath *path, GtkTreeViewColumn *column, ConnectDialog *dialog)
 {
 	GtkTreeIter iter;
-	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (dialog->server_store), &iter, path))
+	if (gtk_tree_model_get_iter (GTK_TREE_MODEL (dialog->server_store), &iter, path)) {
 		gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	}
 }
 
 static void
@@ -104,16 +108,18 @@ connect_dialog_init (ConnectDialog *dialog)
 	GtkTreeIter iter;
 	GSList *netlist;
 	ircnet *net;
+	gchar *path;
 
 	dialog->toplevel = NULL;
 
 	xml = NULL;
-	if (g_file_test ("../../data/connect-dialog.glade", G_FILE_TEST_EXISTS))
-		xml = glade_xml_new ("../../data/connect-dialog.glade", "toplevel", NULL);
-	if (!xml)
-		xml = glade_xml_new (XCHATSHAREDIR "/connect-dialog.glade", "toplevel", NULL);
-	if (!xml)
+	path = locate_data_file ("connect-dialog.glade");
+
+	xml = glade_xml_new (path, "toplevel", NULL);
+	g_free (path);
+	if (!xml) {
 		return;
+	}
 
 #define GW(name) ((dialog->name) = glade_xml_get_widget (xml, #name))
 	GW(toplevel);
@@ -129,7 +135,7 @@ connect_dialog_init (ConnectDialog *dialog)
 	column = gtk_tree_view_column_new_with_attributes ("name", renderer, "text", 0, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (dialog->server_list), column);
 
-	button = gtk_button_new_with_mnemonic (_("_Connect"));
+	button = gtk_button_new_with_mnemonic (_("C_onnect"));
 	gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_OK);
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
