@@ -91,7 +91,7 @@ static void     conversation_panel_print_line         (ConversationPanel      *p
                                                        guchar                  *text,
                                                        int                     len,
                                                        gboolean                indent);
-static void     conversation_panel_lastlog_foreach    (GtkXText               *xtext,
+static void     conversation_panel_lastlog_foreach    (XText                  *xtext,
                                                        guchar                 *text,
                                                        fe_lastlog_info        *info);
 
@@ -168,7 +168,7 @@ struct _ConversationPanelPriv
 
 static GtkActionEntry url_actions[] = {
 	{ "TextURLOpen", GTK_STOCK_OPEN, N_("_Open Link in Browser"), NULL, NULL, G_CALLBACK (open_url) },
-        { "TextURLCopy", GTK_STOCK_COPY, N_("_Copy Link Location"),   NULL, NULL, G_CALLBACK (copy_text) },
+	{ "TextURLCopy", GTK_STOCK_COPY, N_("_Copy Link Location"),   NULL, NULL, G_CALLBACK (copy_text) },
 };
 
 static GtkActionEntry email_actions[] = {
@@ -235,11 +235,11 @@ conversation_panel_init (ConversationPanel *panel)
 	GConfClient       *client;
 
 	panel->priv = g_new0 (ConversationPanelPriv, 1);
-	panel->priv->xtext     = gtk_xtext_new (colors, TRUE);
-	panel->priv->scrollbar = gtk_vscrollbar_new (GTK_XTEXT (panel->priv->xtext)->adj);
+	panel->priv->xtext     = xtext_new ();
+	panel->priv->scrollbar = gtk_vscrollbar_new (xtext_get_adjustment (XTEXT (panel->priv->xtext)));
 	frame                  = gtk_frame_new (NULL);
 
-	panel->priv->buffers            = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) gtk_xtext_buffer_free);
+	panel->priv->buffers            = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, (GDestroyNotify) xtext_buffer_free);
 	panel->priv->timestamp_notifies = g_hash_table_new      (g_direct_hash, g_direct_equal);
 
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
@@ -256,9 +256,9 @@ conversation_panel_init (ConversationPanel *panel)
 
 	action_group = gtk_action_group_new ("TextPopups");
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (action_group, url_actions,   G_N_ELEMENTS (url_actions),   panel);
-	gtk_action_group_add_actions (action_group, email_actions, G_N_ELEMENTS (email_actions), panel);
-	gtk_action_group_add_actions (action_group, dnd_actions,   G_N_ELEMENTS (dnd_actions),   panel);
+	gtk_action_group_add_actions (action_group, url_actions,     G_N_ELEMENTS (url_actions),   panel);
+	gtk_action_group_add_actions (action_group, email_actions,   G_N_ELEMENTS (email_actions), panel);
+	gtk_action_group_add_actions (action_group, dnd_actions,     G_N_ELEMENTS (dnd_actions),   panel);
 	gtk_action_group_add_actions (action_group, default_actions, G_N_ELEMENTS (default_actions),   panel);
 	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
 	g_object_unref (action_group);
@@ -272,7 +272,7 @@ conversation_panel_init (ConversationPanel *panel)
 		gconf_client_get_bool (client, "/apps/xchat/main_window/redundant_nickstamps", NULL);
 	g_object_unref (client);
 
-	g_signal_connect (G_OBJECT (panel), "style_set", G_CALLBACK (style_set_callback), panel);
+	g_signal_connect (G_OBJECT (panel), "style-set", G_CALLBACK (style_set_callback), panel);
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "drag_data_received", G_CALLBACK (drag_data_received), panel);
 	gtk_drag_dest_set (panel->priv->xtext, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 	                   target_table, G_N_ELEMENTS (target_table), GDK_ACTION_COPY | GDK_ACTION_ASK);
@@ -296,8 +296,9 @@ conversation_panel_finalize (GObject *object)
 	g_hash_table_destroy (panel->priv->timestamp_notifies);
 	g_free (panel->priv);
 
-	if (G_OBJECT_CLASS (parent_class)->finalize)
+	if (G_OBJECT_CLASS (parent_class)->finalize) {
 		G_OBJECT_CLASS (parent_class)->finalize (object);
+	}
 }
 
 static void
@@ -306,21 +307,19 @@ conversation_panel_realize (GtkWidget *widget)
 	ConversationPanel *panel;
 	GConfClient       *client;
 
-	if (GTK_WIDGET_CLASS (parent_class)->realize)
+	if (GTK_WIDGET_CLASS (parent_class)->realize) {
 		GTK_WIDGET_CLASS (parent_class)->realize (widget);
+	}
 
 	panel  = CONVERSATION_PANEL (widget);
 	client = gconf_client_get_default ();
 
 	palette_alloc (panel->priv->xtext);
-	gtk_xtext_set_palette           (GTK_XTEXT (panel->priv->xtext), colors);
-	gtk_xtext_set_max_lines         (GTK_XTEXT (panel->priv->xtext), 3000);
-	gtk_xtext_set_show_separator    (GTK_XTEXT (panel->priv->xtext), prefs.show_separator);
-	gtk_xtext_set_indent            (GTK_XTEXT (panel->priv->xtext), prefs.indent_nicks);
-	gtk_xtext_set_max_indent        (GTK_XTEXT (panel->priv->xtext), prefs.max_auto_indent);
-	gtk_xtext_set_thin_separator    (GTK_XTEXT (panel->priv->xtext), prefs.thin_separator);
-	gtk_xtext_set_wordwrap          (GTK_XTEXT (panel->priv->xtext), prefs.wordwrap);
-	gtk_xtext_set_urlcheck_function (GTK_XTEXT (panel->priv->xtext), conversation_panel_check_word);
+	xtext_set_palette           (XTEXT (panel->priv->xtext), colors);
+	xtext_set_max_lines         (XTEXT (panel->priv->xtext), 3000);
+	xtext_set_indent            (XTEXT (panel->priv->xtext), prefs.indent_nicks);
+	xtext_set_max_indent        (XTEXT (panel->priv->xtext), prefs.max_auto_indent);
+	xtext_set_urlcheck_function (XTEXT (panel->priv->xtext), conversation_panel_check_word);
 
 	conversation_panel_set_font       (panel);
 	conversation_panel_set_background (panel);
@@ -330,6 +329,7 @@ conversation_panel_realize (GtkWidget *widget)
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_enter",      G_CALLBACK (conversation_panel_enter_word),   panel);
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_leave",      G_CALLBACK (conversation_panel_leave_word),   panel);
 	g_signal_connect (G_OBJECT (gui.main_window),    "focus-out-event", G_CALLBACK (conversation_panel_lost_focus),   panel);
+	g_signal_connect (G_OBJECT (gui.main_window),    "leave-notify-event", G_CALLBACK (conversation_panel_lost_focus),   panel);
 #endif
 	gconf_client_notify_add (client, "/apps/xchat/main_window/use_sys_fonts",
 	                         (GConfClientNotifyFunc) conversation_panel_font_changed,       panel, NULL, NULL);
@@ -357,13 +357,15 @@ conversation_panel_check_word (GtkWidget *xtext, char *word, int len)
 	url = url_check_word (word, len);
 	if (url == 0 && current_sess) {
 		if (current_sess->type == SESS_DIALOG) {
-			if (strcmp (word, current_sess->channel) == 0)
+			if (strcmp (word, current_sess->channel) == 0) {
 				return WORD_NICK;
+			}
 			return WORD_DIALOG;
 		} else if (((word[0]=='@' || word[0]=='+') && userlist_find (current_sess, word+1)) ||
 			   userlist_find (current_sess, word)) {
-			if (strcmp (word, current_sess->server->nick) != 0)
+			if (strcmp (word, current_sess->server->nick) != 0) {
 				return WORD_NICK;
+			}
 		}
 	}
 	return url;
@@ -372,8 +374,9 @@ conversation_panel_check_word (GtkWidget *xtext, char *word, int len)
 static void
 conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *event, ConversationPanel *panel)
 {
-	if (word == NULL)
+	if (word == NULL) {
 		return;
+	}
 
 	if (event->button == 1) {
 		switch (conversation_panel_check_word (xtext, word, strlen (word))) {
@@ -392,13 +395,14 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				struct session *sess;
 
 				if ((gui.current_session->type == SESS_DIALOG) &&
-				    (strcmp (gui.current_session->channel, word) == 0))
+				    (strcmp (gui.current_session->channel, word) == 0)) {
 					break;
+				}
 
 				sess = find_dialog (gui.current_session->server, word);
-				if (sess)
+				if (sess) {
 					navigation_tree_select_session (gui.server_tree, sess);
-				else {
+				} else {
 					command = g_strdup_printf ("QUERY %s", word);
 					handle_command (gui.current_session, command, 1);
 					g_free (command);
@@ -411,8 +415,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				struct session *sess;
 
 				sess = find_channel (gui.current_session->server, word);
-				if (sess)
+				if (sess) {
 					navigation_tree_select_session (gui.server_tree, sess);
+				}
 				/* Having the channel opened doesn't mean that we're still on it
 				 * (if channel was left) */
 				command = g_strdup_printf ("JOIN %s", word);
@@ -447,8 +452,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 			{
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/TextURLPopup");
-				if (panel->priv->selected_word)
+				if (panel->priv->selected_word) {
 					g_free (panel->priv->selected_word);
+				}
 				panel->priv->selected_word = g_strdup (word);
 				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
@@ -458,8 +464,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				struct User *user;
 				GtkWidget   *menu;
 
-				if (panel->priv->selected_word)
+				if (panel->priv->selected_word) {
 					g_free (panel->priv->selected_word);
+				}
 				panel->priv->selected_word = g_strdup (word);
 
 				if (gui.current_session->type == SESS_CHANNEL) {
@@ -485,8 +492,9 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 			{
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/TextEmailPopup");
-				if (panel->priv->selected_word)
+				if (panel->priv->selected_word) {
 					g_free (panel->priv->selected_word);
+				}
 				panel->priv->selected_word = g_strdup (word);
 				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
@@ -535,12 +543,11 @@ static void
 conversation_panel_enter_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
 {
 	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
-		case WORD_NICK:
-			{
-				panel->priv->tooltip_timeout =
-					g_timeout_add (500, (GSourceFunc) show_user_tooltip, word);
-				break;
-			}
+	case WORD_NICK:
+		{
+			panel->priv->tooltip_timeout = g_timeout_add (500, (GSourceFunc) show_user_tooltip, word);
+			break;
+		}
 	}
 }
 
@@ -562,11 +569,11 @@ static void
 conversation_panel_leave_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
 {
 	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
-		case WORD_NICK:
-			{
-				conversation_panel_remove_tooltip (panel);
-				break;
-			}
+	case WORD_NICK:
+		{
+			conversation_panel_remove_tooltip (panel);
+			break;
+		}
 	}
 }
 
@@ -585,20 +592,22 @@ conversation_panel_set_font (ConversationPanel *panel)
 	gchar       *font = NULL;
 
 	client = gconf_client_get_default ();
-	if (!gconf_client_get_bool(client, "/apps/xchat/main_window/use_sys_fonts", NULL))
+	if (!gconf_client_get_bool(client, "/apps/xchat/main_window/use_sys_fonts", NULL)) {
 		font = gconf_client_get_string (client, "/apps/xchat/main_window/font", NULL);
+	}
 
 	/* Either use_sys_fonts==TRUE, or there is no current font preference.
 	 * In both cases we try the GNOME monospace font. */
-	if (font == NULL)
+	if (font == NULL) {
 		font = gconf_client_get_string (client, "/desktop/gnome/interface/monospace_font_name", NULL);
+	}
 
 	g_object_unref (client);
 
 	if (font == NULL)
 		font = g_strdup ("fixed 11");
 
-	gtk_xtext_set_font (GTK_XTEXT (panel->priv->xtext), font);
+	xtext_set_font (XTEXT (panel->priv->xtext), font);
 
 	g_free (font);
 }
@@ -610,9 +619,9 @@ conversation_panel_font_changed (GConfClient *client, guint cnxn_id, GConfEntry
 
 	conversation_panel_set_font (panel);
 
-	adj = GTK_XTEXT (panel->priv->xtext)->adj;
+	adj = xtext_get_adjustment (XTEXT (panel->priv->xtext));
 	gtk_adjustment_set_value (adj, adj->upper - adj->page_size);
-	gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), FALSE);
+	xtext_refresh (XTEXT (panel->priv->xtext), FALSE);
 }
 
 static void
@@ -624,12 +633,12 @@ conversation_panel_set_background (ConversationPanel *panel)
 	client = gconf_client_get_default ();
 	background_type = gconf_client_get_int (client, "/apps/xchat/main_window/background_type", NULL);
 	if (background_type == 0) {
-		gtk_xtext_set_tint       (GTK_XTEXT (panel->priv->xtext), 0, 0, 0);
-		gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), NULL, FALSE);
+		xtext_set_tint (XTEXT (panel->priv->xtext), 0);
+		xtext_set_background (XTEXT (panel->priv->xtext), NULL, FALSE);
 	} else if (background_type == 1) {
 		gchar *filename = gconf_client_get_string (client, "/apps/xchat/main_window/background_image", NULL);
-		gtk_xtext_set_tint       (GTK_XTEXT (panel->priv->xtext), 0, 0, 0);
-		gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), NULL, FALSE);
+		xtext_set_tint (XTEXT (panel->priv->xtext), 0);
+		xtext_set_background (XTEXT (panel->priv->xtext), NULL, FALSE);
 		if (filename) {
 			GdkPixbuf *pixbuf;
 			pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
@@ -643,7 +652,7 @@ conversation_panel_set_background (ConversationPanel *panel)
 				image = gdk_pixmap_new (NULL, width, height, 24);
 				gdk_draw_pixbuf (image, NULL, pixbuf, 0, 0, 0, 0, width, height, GDK_RGB_DITHER_NONE, 0, 0);
 
-				gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), image, FALSE);
+				xtext_set_background (XTEXT (panel->priv->xtext), image, FALSE);
 				g_object_unref (pixbuf);
 				g_object_unref (image);
 			}
@@ -652,8 +661,8 @@ conversation_panel_set_background (ConversationPanel *panel)
 	} else {
 		float transparency = gconf_client_get_float (client, "/apps/xchat/main_window/background_transparency", NULL);
 		int value = 255 - ((int) (transparency * 255));
-		gtk_xtext_set_tint       (GTK_XTEXT (panel->priv->xtext), value, value, value);
-		gtk_xtext_set_background (GTK_XTEXT (panel->priv->xtext), NULL, TRUE);
+		xtext_set_tint (XTEXT (panel->priv->xtext), value);
+		xtext_set_background (XTEXT (panel->priv->xtext), NULL, TRUE);
 	}
 	g_object_unref (client);
 }
@@ -662,13 +671,13 @@ static void
 conversation_panel_background_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, ConversationPanel *panel)
 {
 	conversation_panel_set_background (panel);
-	gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), TRUE);
+	xtext_refresh (XTEXT (panel->priv->xtext), TRUE);
 }
 
 static void
 timestamps_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, xtext_buffer *buffer)
 {
-	gtk_xtext_set_time_stamp (buffer, gconf_client_get_bool (client, entry->key, NULL));
+	xtext_set_time_stamp (buffer, gconf_client_get_bool (client, entry->key, NULL));
 }
 
 static void
@@ -696,13 +705,15 @@ copy_text (GtkAction *action, ConversationPanel *panel)
 static void
 send_email (GtkAction *action, ConversationPanel *panel)
 {
+	/* FIXME */
 }
 
 static void
 drop_send_files  (GtkAction *action, ConversationPanel *panel)
 {
-	if (panel->priv->current->type != SESS_DIALOG)
+	if (panel->priv->current->type != SESS_DIALOG) {
 		return;
+	}
 	drop_send (panel);
 }
 
@@ -722,8 +733,9 @@ drop_paste_filename  (GtkAction *action, ConversationPanel *panel)
 
 	for (l = panel->priv->dropped_files; l != NULL; l = g_slist_next (l)) {
 		path = g_filename_from_uri (l->data, NULL, NULL);
-		if (path == NULL)
+		if (path == NULL) {
 			path = g_strdup (l->data);
+		}
 
 		if (txt == NULL) {
 			txt = g_strdup (path);
@@ -737,8 +749,9 @@ drop_paste_filename  (GtkAction *action, ConversationPanel *panel)
 		g_free (path);
 	}
 
-	if (panel->priv->current != NULL)
+	if (panel->priv->current != NULL) {
 		handle_multiline (panel->priv->current, txt, TRUE, FALSE);
+	}
 
 	g_free (txt);
 	free_dropped_files (panel);
@@ -954,7 +967,7 @@ style_set_callback (GtkWidget *widget, GtkStyle  *previous_style, void *data)
 	client = gconf_client_get_default ();
 	color_scheme = gconf_client_get_int (client, "/apps/xchat/irc/color_scheme", NULL);
 	g_object_unref (client);
-	if (color_scheme == 0) {
+	if (color_scheme == 3) {
 		load_colors (color_scheme);
 		load_palette (color_scheme);
 	}
@@ -971,8 +984,8 @@ void
 conversation_panel_update_colors (ConversationPanel *panel)
 {
 	palette_alloc (panel->priv->xtext);
-	gtk_xtext_set_palette (GTK_XTEXT (panel->priv->xtext), colors);
-	gtk_xtext_refresh     (GTK_XTEXT (panel->priv->xtext), FALSE);
+	xtext_set_palette (XTEXT (panel->priv->xtext), colors);
+	xtext_refresh     (XTEXT (panel->priv->xtext), FALSE);
 }
 
 void
@@ -982,10 +995,10 @@ conversation_panel_add_session (ConversationPanel *panel, struct session *sess,
 	xtext_buffer *buffer;
 	gint          notify;
 
-	buffer = gtk_xtext_buffer_new (GTK_XTEXT (panel->priv->xtext));
+	buffer = xtext_buffer_new (XTEXT (panel->priv->xtext));
 
 	client = gconf_client_get_default ();
-	gtk_xtext_set_time_stamp (buffer, gconf_client_get_bool (client, "/apps/xchat/irc/showtimestamps", NULL));
+	xtext_set_time_stamp (buffer, gconf_client_get_bool (client, "/apps/xchat/irc/showtimestamps", NULL));
 	notify = gconf_client_notify_add (client, "/apps/xchat/irc/showtimestamps",
 	                                  (GConfClientNotifyFunc) timestamps_changed, buffer, NULL, NULL);
 	g_object_unref (client);
@@ -1004,8 +1017,12 @@ conversation_panel_set_current (ConversationPanel *panel, struct session *sess)
 	xtext_buffer *buffer;
 
 	panel->priv->current = sess;
-	buffer = g_hash_table_lookup (panel->priv->buffers, sess);
-	gtk_xtext_buffer_show (GTK_XTEXT (panel->priv->xtext), buffer, TRUE);
+	if (sess) {
+		buffer = g_hash_table_lookup (panel->priv->buffers, sess);
+		xtext_buffer_show (XTEXT (panel->priv->xtext), buffer, TRUE);
+	} else {
+		xtext_buffer_show (XTEXT (panel->priv->xtext), XTEXT (panel->priv->xtext)->orig_buffer, TRUE);
+	}
 
 #ifdef HAVE_LIBSEXY
 	conversation_panel_remove_tooltip (panel);
@@ -1056,7 +1073,7 @@ conversation_panel_save_current (ConversationPanel *panel)
 			g_error_free (error);
 		} else {
 			gint fd = g_io_channel_unix_get_fd (file);
-			gtk_xtext_save (GTK_XTEXT (panel->priv->xtext), fd);
+			xtext_save (XTEXT (panel->priv->xtext), fd);
 			g_io_channel_shutdown (file, TRUE, &error);
 
 			if (error) {
@@ -1077,8 +1094,8 @@ conversation_panel_clear (ConversationPanel *panel, struct session *sess)
 	xtext_buffer *buffer;
 
 	buffer = g_hash_table_lookup (panel->priv->buffers, sess);
-	gtk_xtext_clear (buffer);
-	gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), FALSE);
+	xtext_clear (buffer);
+	xtext_refresh (XTEXT (panel->priv->xtext), FALSE);
 }
 
 static void
@@ -1101,7 +1118,7 @@ conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, g
 		memcpy (new_text, stamp, stamp_size);
 		g_free (stamp);
 		memcpy (new_text + stamp_size, text, len);
-		gtk_xtext_append (buffer, new_text, len + stamp_size);
+		xtext_append (buffer, new_text, len + stamp_size);
 		g_free (new_text);
 		return;
 	}
@@ -1110,20 +1127,17 @@ conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, g
 	if (tab && tab < (text + len)) {
 		leftlen = tab - text;
 
-		if(!panel->priv->redundant_nickstamps && strncmp (buffer->laststamp, text, leftlen) == 0)
-		{
+		if(!panel->priv->redundant_nickstamps && strncmp (buffer->laststamp, text, leftlen) == 0) {
 			text = tab+1;
 			len -= leftlen;
-			gtk_xtext_append_indent (buffer, 0, 0, text, len);
-		}
-		else
-		{
+			xtext_append_indent (buffer, 0, 0, text, len);
+		} else {
 			strncpy (buffer->laststamp, text, leftlen);
 			buffer->laststamp[leftlen]=0;
-			gtk_xtext_append_indent (buffer, text, leftlen, tab + 1, strlen (text) - leftlen - 1);
+			xtext_append_indent (buffer, text, leftlen, tab + 1, strlen (text) - leftlen - 1);
 		}
 	} else {
-		gtk_xtext_append_indent (buffer, 0, 0, text, len);
+		xtext_append_indent (buffer, 0, 0, text, len);
 	}
 }
 
@@ -1135,6 +1149,9 @@ conversation_panel_print (ConversationPanel *panel, struct session *sess, guchar
 	int           len       = 0;
 
 	buffer = g_hash_table_lookup (panel->priv->buffers, sess);
+	if (buffer == NULL) {
+		return;
+	}
 
 	/* split the text into separate lines */
 	while (1) {
@@ -1176,7 +1193,7 @@ conversation_panel_remove_session (ConversationPanel *panel, struct session *ses
 }
 
 static void
-conversation_panel_lastlog_foreach (GtkXText *xtext, guchar *text, fe_lastlog_info *info)
+conversation_panel_lastlog_foreach (XText *xtext, guchar *text, fe_lastlog_info *info)
 {
 	if (nocasestrstr (text, info->sstr)) {
 		conversation_panel_print (info->panel, info->sess, text, prefs.indent_nicks);
@@ -1191,7 +1208,7 @@ conversation_panel_lastlog (ConversationPanel *panel, struct session *sess, stru
 	buffer  = g_hash_table_lookup (panel->priv->buffers, sess);
 	lbuffer = g_hash_table_lookup (panel->priv->buffers, lsess);
 
-	if (gtk_xtext_is_empty (buffer)) {
+	if (xtext_is_empty (buffer)) {
 		conversation_panel_print (panel, lsess, _("Search buffer is empty.\n"), TRUE);
 	} else {
 		fe_lastlog_info info;
@@ -1199,21 +1216,21 @@ conversation_panel_lastlog (ConversationPanel *panel, struct session *sess, stru
 		info.sess  = lsess;
 		info.sstr  = sstr;
 
-		gtk_xtext_foreach (buffer, (GtkXTextForeach) conversation_panel_lastlog_foreach, &info);
+		xtext_foreach (buffer, (XTextForeach) conversation_panel_lastlog_foreach, &info);
 	}
 }
 
 void
 conversation_panel_clear_selection (ConversationPanel *panel)
 {
-	gtk_xtext_selection_clear_full (GTK_XTEXT (panel->priv->xtext)->buffer);
-        gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), TRUE);
+	xtext_selection_clear_full (XTEXT (panel->priv->xtext)->buffer);
+        xtext_refresh (XTEXT (panel->priv->xtext), TRUE);
 }
 
 gpointer
 conversation_panel_search (ConversationPanel *panel, const gchar *text, gpointer start, gboolean casem, gboolean reverse)
 {
-	return gtk_xtext_search (GTK_XTEXT (panel->priv->xtext), text, start, casem, reverse);
+	return xtext_search (XTEXT (panel->priv->xtext), text, start, casem, reverse);
 }
 
 void
@@ -1222,13 +1239,15 @@ conversation_panel_page_up (ConversationPanel *panel)
 	GtkAdjustment *adj;
 	int end, value;
 
-	adj = GTK_XTEXT (panel->priv->xtext)->adj;
+	adj = xtext_get_adjustment (XTEXT (panel->priv->xtext));
 	end = adj->upper - adj->lower - adj->page_size;
 	value = adj->value - (adj->page_size - 1);
-	if (value < 0)
+	if (value < 0) {
 		value = 0;
-	if (value > end)
+	}
+	if (value > end) {
 		value = end;
+	}
 	gtk_adjustment_set_value (adj, value);
 }
 
@@ -1238,27 +1257,28 @@ conversation_panel_page_down (ConversationPanel *panel)
 	GtkAdjustment *adj;
 	int value, end;
 
-	adj = GTK_XTEXT (panel->priv->xtext)->adj;
+	adj = xtext_get_adjustment (XTEXT (panel->priv->xtext));
 	end = adj->upper - adj->lower - adj->page_size;
 	value = adj->value + (adj->page_size - 1);
-	if (value < 0)
+	if (value < 0) {
 		value = 0;
-	if (value > end)
+	}
+	if (value > end) {
 		value = end;
+	}
 	gtk_adjustment_set_value (adj, value);
 }
 
 void
 conversation_panel_copy_selection (ConversationPanel *panel)
 {
-	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (panel), GDK_SELECTION_CLIPBOARD);
-	gtk_xtext_copy_selection (GTK_XTEXT (panel->priv->xtext), clipboard);
+	xtext_copy_selection (XTEXT (panel->priv->xtext));
 }
 
 void
 conversation_panel_queue_tdraw (ConversationPanel *panel)
 {
-	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
+	if (XTEXT (panel->priv->xtext)->transparent) {
 		gtk_widget_queue_draw (panel->priv->xtext);
 	}
 }
@@ -1266,8 +1286,8 @@ conversation_panel_queue_tdraw (ConversationPanel *panel)
 static void
 redraw_transparency (ConversationPanel *panel)
 {
-	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
-		gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), 1);
+	if (XTEXT (panel->priv->xtext)->transparent) {
+		xtext_refresh (XTEXT (panel->priv->xtext), 1);
 	}
 }
 
@@ -1279,10 +1299,12 @@ root_event_cb (GdkXEvent *xev, GdkEventProperty *event, ConversationPanel *panel
 	XEvent *xevent = (XEvent *)xev;
 
 	if (xevent->type == PropertyNotify) {
-		if (at == None)
+		if (at == None) {
 			at = XInternAtom (xevent->xproperty.display, "_XROOTPMAP_ID", True);
-		if (at == xevent->xproperty.atom)
+		}
+		if (at == xevent->xproperty.atom) {
 			redraw_transparency (panel);
+		}
 	}
 	return GDK_FILTER_CONTINUE;
 }
@@ -1291,11 +1313,11 @@ root_event_cb (GdkXEvent *xev, GdkEventProperty *event, ConversationPanel *panel
 void
 conversation_panel_check_marker_visibility (ConversationPanel *panel)
 {
-	gtk_xtext_check_marker_visibility (GTK_XTEXT (panel->priv->xtext));
+	xtext_check_marker_visibility (XTEXT (panel->priv->xtext));
 }
 
 void
 conversation_panel_set_show_marker (ConversationPanel *panel, gboolean show_marker)
 {
-	gtk_xtext_set_show_marker (GTK_XTEXT (panel->priv->xtext), show_marker);
+	xtext_set_show_marker (XTEXT (panel->priv->xtext), show_marker);
 }
