@@ -61,7 +61,7 @@ static void      show_context_menu (GtkWidget *treeview, GdkEventButton *event);
 static gboolean  click (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 static gboolean  declick (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 static gboolean  popup_menu (GtkWidget *treeview, gpointer user_data);
-static void      navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_data);
+static void      navigation_selection_changed (GtkTreeSelection *treeselection, NavTree *navtree);
 static void      row_expanded (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
 static void      row_collapsed (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
 static void      on_server_information (GtkAction *action, gpointer data);
@@ -107,7 +107,17 @@ static GtkToggleActionEntry toggle_action_entries[] = {
 	{"ChannelAutoJoin",   NULL, N_("_Auto-join on connect"),    "", NULL, G_CALLBACK (on_channel_autojoin), FALSE},
 };
 
-static GtkActionGroup *action_group;
+enum
+{
+	COLUMN_ICON,
+	COLUMN_NAME,
+	COLUMN_SESSION,
+	COLUMN_STATUS,
+	COLUMN_COLOR,
+	COLUMN_REFCOUNT,
+	COLUMN_CONNECTED,
+	N_COLUMNS
+};
 
 GType
 navigation_tree_get_type (void)
@@ -141,17 +151,19 @@ navigation_tree_init (NavTree *navtree)
 	GtkCellRenderer *icon_renderer, *text_renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
+	GtkActionGroup *action_group;
+	GtkAction *action;
 
 	g_object_set ((gpointer) navtree, "headers-visible", FALSE, NULL);
 	navtree->current_rowref = NULL;
 	navtree->model = NULL;
 	navtree->selection_changed_id = 0;
 
-	action_group = gtk_action_group_new ("NavigationContext");
+	navtree->action_group = action_group = gtk_action_group_new ("NavigationContext");
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (action_group, action_entries, G_N_ELEMENTS (action_entries), NULL);
 	gtk_action_group_add_toggle_actions (action_group, toggle_action_entries, G_N_ELEMENTS (toggle_action_entries), NULL);
-	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
+	gtk_ui_manager_insert_action_group (gui.manager, action_group, -1);
 	g_object_unref (action_group);
 
 	/* This sets up all our columns. */
@@ -160,10 +172,10 @@ navigation_tree_init (NavTree *navtree)
 	text_renderer = gtk_cell_renderer_text_new ();
 	/* Icon columns. */
 	gtk_tree_view_column_pack_start (column, icon_renderer, FALSE);
-	gtk_tree_view_column_set_attributes (column, icon_renderer, "pixbuf", 0, NULL);
+	gtk_tree_view_column_set_attributes (column, icon_renderer, "icon-name", COLUMN_ICON, NULL);
 	/* text columns. */
 	gtk_tree_view_column_pack_start (column, text_renderer, TRUE);
-	gtk_tree_view_column_set_attributes (column, text_renderer, "text", 1, "foreground-gdk", 4, NULL);
+	gtk_tree_view_column_set_attributes (column, text_renderer, "text", COLUMN_NAME, "foreground-gdk", COLUMN_COLOR, NULL);
 	/* Add the column to the TreeView. */
 	gtk_tree_view_append_column (GTK_TREE_VIEW (navtree), column);
 
@@ -172,7 +184,7 @@ navigation_tree_init (NavTree *navtree)
 	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
 
 	/* Connect the callbacks. */
-	navtree->selection_changed_id = g_signal_connect (G_OBJECT (select), "changed", G_CALLBACK (navigation_selection_changed), NULL);
+	navtree->selection_changed_id = g_signal_connect (G_OBJECT (select), "changed", G_CALLBACK (navigation_selection_changed), navtree);
 	g_signal_connect (G_OBJECT (navtree), "row-expanded", G_CALLBACK (row_expanded), NULL);
 	g_signal_connect (G_OBJECT (navtree), "row-collapsed", G_CALLBACK (row_collapsed), NULL);
 	g_signal_connect (G_OBJECT (navtree), "button_press_event", G_CALLBACK (click), NULL);
@@ -182,6 +194,10 @@ navigation_tree_init (NavTree *navtree)
 	/* Initialize our row references. */
 	navtree->last_server = NULL;
 	navtree->last_channel = NULL;
+
+	/* Temporarily disable menu items */
+	action = gtk_action_group_get_action (action_group, "ChannelBans");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 static void
@@ -207,8 +223,10 @@ static void
 navigation_tree_finalize (GObject *object)
 {
 	NavTree *navtree = (NavTree *) object;
-	gtk_tree_row_reference_free (navtree->current_rowref);
-	navtree->current_rowref = NULL;
+	if (NULL == navtree->current_rowref) {
+		gtk_tree_row_reference_free (navtree->current_rowref);
+		navtree->current_rowref = NULL;
+	}
 }
 
 /* New NavTree. */
@@ -231,7 +249,8 @@ navigation_tree_new (NavModel *model)
 void
 navigation_tree_create_new_network_entry (NavTree *navtree, struct session *sess)
 {
-	GtkWidget *menuitem, *button;
+	GtkWidget *button;
+	GtkAction *action;
 
 	navigation_model_add_new_network (navtree->model, sess);
 
@@ -250,16 +269,15 @@ navigation_tree_create_new_network_entry (NavTree *navtree, struct session *sess
 	gtk_widget_set_sensitive (button, FALSE);
 
 	/* Make the topic button insensitive. */
-	menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionChangeTopicItem");
-	if (menuitem == NULL)
-		g_warning ("can't disable topic change menu item");
-	gtk_widget_set_sensitive (menuitem, FALSE);
+	action = gtk_action_group_get_action (gui.action_group, "DiscussionChangeTopic");
+	gtk_action_set_sensitive (action, FALSE);
 }
 
 void
 navigation_tree_create_new_channel_entry (NavTree *navtree, struct session *sess, gboolean focus)
 {
-	GtkWidget *menuitem, *button;
+	GtkWidget *button;
+	GtkAction *action;
 	ircnet *net;
 
 	navigation_model_add_new_channel (navtree->model, sess);
@@ -278,19 +296,15 @@ navigation_tree_create_new_channel_entry (NavTree *navtree, struct session *sess
 
 		topic_label_set_current (TOPIC_LABEL (gui.topic_label), sess);
 		net = sess->server->network;
-		if (net == NULL)
-			rename_main_window (NULL, sess->channel);
-		else
-			rename_main_window (net->name, sess->channel);
+		rename_main_window (net == NULL ? NULL : net->name, sess->channel);
 
 		button = glade_xml_get_widget (gui.xml, "close discussion");
-		if (sess->type == SESS_CHANNEL)
+		if (sess->type == SESS_CHANNEL) {
 			gtk_widget_set_sensitive (button, TRUE);
-		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionChangeTopicItem");
-		if (menuitem == NULL)
-			g_warning ("can't access topic change menu item");
-		else
-			gtk_widget_set_sensitive (menuitem, sess->type == SESS_CHANNEL);
+		}
+
+		action = gtk_action_group_get_action (gui.action_group, "DiscussionChangeTopic");
+		gtk_action_set_sensitive (action, sess->type == SESS_CHANNEL);
 
 		/* Set our nick. */
 		set_nickname (sess->server, NULL);
@@ -341,27 +355,20 @@ navigation_tree_remove_server (NavTree *navtree, struct session *sess)
 		GtkTreeModel *sorted = gtk_tree_view_get_model (GTK_TREE_VIEW (navtree));
 		GtkTreeIter  iter;
 
-		gtk_tree_model_get_iter_first (sorted, &iter);
+		navigation_model_remove (navtree->model, sess);
 
-		if (gtk_tree_model_iter_next (sorted, &iter)) {
+		if (gtk_tree_model_get_iter_first (sorted, &iter)) {
 			/* There's another server after the first one. */
 			gtk_tree_selection_select_iter (select, &iter);
-			navigation_model_remove (navtree->model, sess);
-
-			/* We rely on the above select_iter call triggering the selection 'changed' event to do book-keeping */
 		} else {
-			/* The first server is the only server. */
-			GtkTreeModel *store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (sorted));
-
-			gtk_tree_model_get_iter_first (store, &iter);
-			navigation_tree_server_rm_chans (navtree, &iter);
-			gtk_tree_model_get_iter_first (store, &iter);
-			gtk_tree_store_set (GTK_TREE_STORE (store), &iter, 1, _("<none>"), -1);
+			gtk_tree_selection_unselect_all (select);
 		}
+
+		/* We rely on the above select_iter call triggering the selection 'changed' event to do book-keeping */
 	} else {
 		/* This isn't the first server. Select the server above it and remove it. */
-		gtk_tree_selection_select_path (select, path);
 		navigation_model_remove (navtree->model, sess);
+		gtk_tree_selection_select_path (select, path);
 	}
 
 	navigation_tree_update_refs (navtree);
@@ -608,8 +615,8 @@ navigation_tree_select_prev_channel (NavTree *navtree)
 		/* If a channel is selected... */
 		if (!gtk_tree_path_prev (path)) {
 			/* If it's the first channel on the server, pretend we've got
-		 	 * that server selected.
-		 	 */
+			 * that server selected.
+			 */
 			depth = 1;
 			gtk_tree_path_up (path);
 		}
@@ -751,8 +758,9 @@ navigation_tree_select_prev_network (NavTree *navtree)
 		GtkTreeIter current, previous;
 		gtk_tree_model_get_iter_first (model, &current);
 		previous = current;
-		while (gtk_tree_model_iter_next (model, &current))
+		while (gtk_tree_model_iter_next (model, &current)) {
 			previous = current;
+		}
 		/* Free the old path and set it to the iter that points to the last entry. */
 		gtk_tree_path_free (path);
 		path = gtk_tree_model_get_path (model, &previous);
@@ -777,8 +785,9 @@ navigation_tree_get_selected_session (gboolean *connected)
 	select = gtk_tree_view_get_selection (treeview);
 	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
 		gtk_tree_model_get (model, &iter, 2, &s, 6, &temp, -1);
-		if (connected)
+		if (connected) {
 			*connected = temp;
+		}
 		return s;
 	}
 	return NULL;
@@ -787,9 +796,9 @@ navigation_tree_get_selected_session (gboolean *connected)
 static void
 navigation_tree_update_refs (NavTree *navtree)
 {
-	GtkTreeModel*	model = GTK_TREE_MODEL (navtree->model->sorted);
-	GtkTreeIter	iter;
-	GtkTreePath*	path;
+	GtkTreeModel* model = GTK_TREE_MODEL (navtree->model->sorted);
+	GtkTreeIter   iter;
+	GtkTreePath*  path;
 
 	/* If there is nothing in the tree, set the references to NULL. */
 	if (!gtk_tree_model_get_iter_first (model, &iter)) {
@@ -801,26 +810,29 @@ navigation_tree_update_refs (NavTree *navtree)
 	path = gtk_tree_path_new_first ();
 
 	/*** Find the last network in the treeview. ***/
-	if (navtree->last_server)
+	if (navtree->last_server) {
 		/* Free the current reference if there is one. */
 		gtk_tree_row_reference_free (navtree->last_server);
+	}
 
 	/* Find the last server. */
-	while (gtk_tree_model_iter_next (model, &iter))
+	while (gtk_tree_model_iter_next (model, &iter)) {
 		gtk_tree_path_next (path);
+	}
 
 	/* Set the reference. */
 	navtree->last_server = gtk_tree_row_reference_new (model, path);
 
 	/*** Find the last visible channel in the treeview. ***/
 
-	if (navtree->last_channel)
+	if (navtree->last_channel) {
 		/* Free the current reference if there is one. */
 		gtk_tree_row_reference_free (navtree->last_channel);
+	}
 
 	/* Find the server furthest down the list with visible children. */
 	do {
-		gint	children = 0;
+		gint children = 0;
 
 		gtk_tree_model_get_iter (model, &iter, path);
 		children = gtk_tree_model_iter_n_children (model, &iter);
@@ -834,16 +846,17 @@ navigation_tree_update_refs (NavTree *navtree)
 		}
 	} while (gtk_tree_path_prev (path));
 
-	if (gtk_tree_path_get_depth (path) == 1)
+	if (gtk_tree_path_get_depth (path) == 1) {
 		/* If the path is still pointing to servers, we must not be
 		 * connected to any channels. Set this reference to NULL.
 		 */
 		navtree->last_channel = NULL;
-	else
+	} else {
 		/* At this point, path refers to the last channel in the
 		 * treeview.
 		 */
 		navtree->last_channel = gtk_tree_row_reference_new (model, path);
+	}
 
 	gtk_tree_path_free (path);
 }
@@ -857,7 +870,11 @@ navigation_tree_set_channel_name_iterate (GtkTreeModel *model, GtkTreePath *path
 	gtk_tree_model_get (model, iter, 2, &s, -1);
 	if (s == data) {
 		struct session *sess = s;
-		gtk_tree_store_set (GTK_TREE_STORE (model), iter, 1, (sess->channel), 4, NULL, 6, TRUE, -1);
+		gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+		                    COLUMN_NAME,      (sess->channel),
+		                    COLUMN_COLOR,     NULL,
+		                    COLUMN_CONNECTED, TRUE,
+		                    -1);
 		return TRUE;
 	}
 	return FALSE;
@@ -919,6 +936,7 @@ navigation_context (GtkWidget *treeview, session *selected)
 static GtkWidget *
 server_context (GtkWidget *treeview, session *selected)
 {
+	NavTree *navtree = NAVTREE (treeview);
 	GtkWidget *menu;
 	GtkAction *action;
 
@@ -930,16 +948,17 @@ server_context (GtkWidget *treeview, session *selected)
 		 * FIXME: it would be really nice to be able to configure this server
 		 * as part of a network here.
 		 */
-		action = gtk_action_group_get_action (action_group, "ServerAutoConnect");
+		action = gtk_action_group_get_action (navtree->action_group, "ServerAutoConnect");
 		gtk_action_set_sensitive (action, FALSE);
 	} else if (selected != NULL) {
 		/* check if the network is in the auto-connect list */
-		action = gtk_action_group_get_action (action_group, "ServerAutoConnect");
+		action = gtk_action_group_get_action (navtree->action_group, "ServerAutoConnect");
 		gtk_action_set_sensitive (action, TRUE);
-		if (((ircnet *)selected->server->network)->flags & FLAG_AUTO_CONNECT)
+		if (((ircnet *)selected->server->network)->flags & FLAG_AUTO_CONNECT) {
 			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-		else
+		} else {
 			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+		}
 	}
 	return menu;
 }
@@ -947,6 +966,7 @@ server_context (GtkWidget *treeview, session *selected)
 static GtkWidget *
 channel_context (GtkWidget *treeview, session *selected)
 {
+	NavTree *navtree = NAVTREE (treeview);
 	GtkWidget *menu;
 	GtkAction *action;
 	ircnet *network;
@@ -959,17 +979,18 @@ channel_context (GtkWidget *treeview, session *selected)
 	g_return_val_if_fail (menu != NULL, NULL);
 
 	/* check if the channel is in the auto-join list */
-	action = gtk_action_group_get_action (action_group, "ChannelAutoJoin");
+	action = gtk_action_group_get_action (navtree->action_group, "ChannelAutoJoin");
 	network = selected->server->network;
 
 	if (network == NULL) {
 		gtk_action_set_sensitive (action, FALSE);
 	} else {
 		gtk_action_set_sensitive (action, TRUE);
-		if (channel_is_autojoin (network, selected->channel))
+		if (channel_is_autojoin (network, selected->channel)) {
 			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-		else
+		} else {
 			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+		}
 	}
 
 	return menu;
@@ -995,8 +1016,9 @@ show_context_menu (GtkWidget *treeview, GdkEventButton *event)
 	GtkWidget *menu;
 	session *s;
 
-	if (NAVTREE (treeview)->current_rowref == NULL)
+	if (NAVTREE (treeview)->current_rowref == NULL) {
 		return;
+	}
 
 	model = gtk_tree_row_reference_get_model (NAVTREE (treeview)->current_rowref);
 	path = gtk_tree_row_reference_get_path (NAVTREE (treeview)->current_rowref);
@@ -1004,19 +1026,20 @@ show_context_menu (GtkWidget *treeview, GdkEventButton *event)
 	gtk_tree_path_free(path);
 	gtk_tree_model_get (model, &iter, 2, &s, -1);
 
-	if (s == NULL)
+	if (s == NULL) {
 		return;
+	}
 
-	menu = navigation_context (treeview, s); /* FIXME */	
+	menu = navigation_context (treeview, s); /* FIXME */
 	g_return_if_fail (menu != NULL);
 
 	if (event != NULL) {
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-				event->button, event->time);
+		                event->button, event->time);
 	} else {
 		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
-				menu_position_under_tree_view, treeview,
-				0, gtk_get_current_event_time ());
+		                menu_position_under_tree_view, treeview,
+		                0, gtk_get_current_event_time ());
 
 		gtk_menu_shell_select_first (GTK_MENU_SHELL (menu), FALSE);
 	}
@@ -1029,12 +1052,14 @@ click (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 	GtkTreePath *path;
 	GtkTreeSelection *select;
 
-	if (!event)
+	if (!event) {
 		return FALSE;
+	}
 
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-	if (gtk_tree_selection_get_selected (select, NULL, NULL) == FALSE)
+	if (gtk_tree_selection_get_selected (select, NULL, NULL) == FALSE) {
 		return FALSE;
+	}
 
 	if (event->button == 3) {
 		if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (treeview), event->x, event->y, &path, 0, 0, 0)) {
@@ -1071,22 +1096,26 @@ popup_menu (GtkWidget *treeview, gpointer user_data)
 }
 
 static void
-navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_data)
+navigation_selection_changed (GtkTreeSelection *treeselection, NavTree *navtree)
 {
 	GtkTreeIter   iter;
 	GtkTreeIter   newiter;
 	GtkTreeModel *model;
 	GtkTreeModel *store;
 	GtkWidget    *button;
+	GtkAction    *action;
 	gpointer     *s;
 	session      *sess;
 
-	if (gui.server_tree == NULL)
+	if (gui.server_tree == NULL) {
 		return;
+	}
 
-
-	if (gui.server_tree->current_rowref != NULL)
+	if (gui.server_tree->current_rowref != NULL) {
 		navigation_model_rowref_deref (gui.server_tree->current_rowref);
+		gtk_tree_row_reference_free (gui.server_tree->current_rowref);
+		gui.server_tree->current_rowref = NULL;
+	}
 
 	/* If find bar is open, hide it */
 	find_bar_close (FIND_BAR (gui.find_bar));
@@ -1094,21 +1123,17 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 	/* XXX: This sets model to be the GtkTreeModelSort used by the NavTree, it is
 	 *      not a GtkTreeModel. The iter is for that ModelSort.
 	 */
-	if (gtk_tree_selection_get_selected (treeselection, &model, &iter) && gui.current_session) {
-		GtkWidget *menuitem;
+	if (gtk_tree_selection_get_selected (treeselection, &model, &iter)) {
 		GtkTreePath *path;
 
 		path = gtk_tree_model_get_path (model, &iter);
 
-		/* Update current_rowref. */
-		if (gui.server_tree->current_rowref) {
-			gtk_tree_row_reference_free (gui.server_tree->current_rowref);
-		}
-
-		gui.server_tree->current_rowref = gtk_tree_row_reference_new(model, path);
+		gui.server_tree->current_rowref = gtk_tree_row_reference_new (model, path);
 		navigation_model_rowref_ref (gui.server_tree->current_rowref);
 		gtk_tree_selection_get_selected (treeselection, &model, &iter);
 
+		gtk_tree_path_free (path);
+
 		/* Get the session for the new selection. */
 		gtk_tree_model_get (model, &iter, 2, &s, -1);
 		sess = (session *) s;
@@ -1123,7 +1148,7 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 
 		/* Change the model in the userlist to the one for this session. */
 		gtk_tree_view_set_model (GTK_TREE_VIEW (gui.userlist),
-				         GTK_TREE_MODEL (userlist_get_store (u, sess)));
+		                         GTK_TREE_MODEL (userlist_get_store (u, sess)));
 
 		/* Set our nick. */
 		set_nickname (sess->server, NULL);
@@ -1137,26 +1162,29 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 		}
 
 		/* Set widget sensitivity based on type of the current "tab" */
-		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionChangeTopicItem");
-		if (menuitem == NULL)
-			g_warning ("can't disable topic change menu item");
-		else
-			gtk_widget_set_sensitive (menuitem, sess->type == SESS_CHANNEL);
+		action = gtk_action_group_get_action (gui.action_group, "DiscussionChangeTopic");
+		gtk_action_set_sensitive (action, sess->type == SESS_CHANNEL);
+
+		/* FIXME make this a proxy widget of DiscussionClose */
 		button = glade_xml_get_widget (gui.xml, "close discussion");
 		gtk_widget_set_sensitive (button, sess->type != SESS_SERVER);
 
-		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionUsersItem");
-		gtk_widget_set_sensitive (menuitem, sess->type != SESS_SERVER);
-		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionLeaveItem");
-		gtk_widget_set_sensitive (menuitem, sess->type != SESS_SERVER);
-		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionCloseItem");
-		gtk_widget_set_sensitive (menuitem, sess->type != SESS_SERVER);
+		action = gtk_action_group_get_action (gui.action_group, "DiscussionUsers");
+		gtk_action_set_sensitive (action, sess->type != SESS_SERVER);
+		action = gtk_action_group_get_action (gui.action_group, "DiscussionLeave");
+		gtk_action_set_sensitive (action, sess->type != SESS_SERVER);
+		action = gtk_action_group_get_action (gui.action_group, "DiscussionClose");
+		gtk_action_set_sensitive (action, sess->type != SESS_SERVER);
+
 		gtk_widget_set_sensitive (gui.userlist_toggle, sess->type == SESS_CHANNEL);
 
 		/* remove any icon that exists */
 		store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
 		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &newiter, &iter);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &newiter, 0, NULL, 3, 0, -1);
+		gtk_tree_store_set (GTK_TREE_STORE (store), &newiter,
+		                    COLUMN_ICON, NULL,
+		                    COLUMN_STATUS, 0,
+		                    -1);
 
 		conversation_panel_set_current (CONVERSATION_PANEL (gui.conversation_panel), sess);
 		topic_label_set_current        (TOPIC_LABEL        (gui.topic_label),        sess);
@@ -1164,13 +1192,24 @@ navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_dat
 		status_bar_set_current         (STATUS_BAR         (gui.status_bar),         sess->server);
 
 		gtk_widget_grab_focus (gui.text_entry);
-		gtk_editable_set_position (GTK_EDITABLE (gui.text_entry), -1);
 
 		/* Set the label of the user list button */
 		userlist_set_user_button (u, sess);
 
 		/* Emit "focus tab" event */
 		plugin_emit_dummy_print (sess, "Focus Tab");
+
+		gtk_widget_set_sensitive (gui.nick_button, TRUE);
+	} else {
+		gui.current_session = NULL;
+		gtk_window_set_title (GTK_WINDOW (gui.main_window), _("XChat-GNOME IRC Chat"));
+		gtk_button_set_label (GTK_BUTTON (gui.nick_button), "");
+		gtk_widget_set_sensitive (gui.nick_button, FALSE);
+
+		conversation_panel_set_current (CONVERSATION_PANEL (gui.conversation_panel), NULL);
+		topic_label_set_current        (TOPIC_LABEL        (gui.topic_label),        NULL);
+		text_entry_set_current         (TEXT_ENTRY         (gui.text_entry),         NULL);
+		status_bar_set_current         (STATUS_BAR         (gui.status_bar),         NULL);
 	}
 }
 
@@ -1179,8 +1218,9 @@ row_expanded (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpoin
 {
 	GtkTreePath *current_path;
 
-	if (NAVTREE (treeview)->current_rowref == NULL)
+	if (NAVTREE (treeview)->current_rowref == NULL) {
 		return;
+	}
 	current_path = gtk_tree_row_reference_get_path (NAVTREE (treeview)->current_rowref);
 
 	/* If we had something selected before the row was collapsed make sure it gets selected. */
@@ -1216,13 +1256,13 @@ navigation_model_get_type (void)
 	if (!navigation_model_type) {
 		static const GTypeInfo navigation_model_info = {
 			sizeof (NavTreeClass),
-			NULL,		/* base init. */
-			NULL,		/* base finalize. */
+			NULL, /* base init. */
+			NULL, /* base finalize. */
 			(GClassInitFunc) navigation_model_class_init,
-			NULL,		/* class_finalize. */
-			NULL,		/* class_data. */
+			NULL, /* class_finalize. */
+			NULL, /* class_data. */
 			sizeof (NavModel),
-			0,			/* n_preallocs. */
+			0, /* n_preallocs. */
 			(GInstanceInitFunc) navigation_model_init,
 		};
 
@@ -1235,13 +1275,13 @@ navigation_model_get_type (void)
 static void
 navigation_model_init (NavModel *navmodel)
 {
-	navmodel->store = gtk_tree_store_new (7, GDK_TYPE_PIXBUF, /* status image */
-	                                         G_TYPE_STRING,   /* name */
-	                                         G_TYPE_POINTER,  /* session pointer */
-	                                         G_TYPE_INT,      /* status # (for tracking highest state) */
-	                                         GDK_TYPE_COLOR,  /* status color (disconnected, etc) */
-	                                         G_TYPE_INT,      /* reference count */
-	                                         G_TYPE_BOOLEAN); /* connected */
+	navmodel->store = gtk_tree_store_new (N_COLUMNS, G_TYPE_STRING,   /* status image */
+	                                                 G_TYPE_STRING,   /* name */
+	                                                 G_TYPE_POINTER,  /* session pointer */
+	                                                 G_TYPE_INT,      /* status # (for tracking highest state) */
+	                                                 GDK_TYPE_COLOR,  /* status color (disconnected, etc) */
+	                                                 G_TYPE_INT,      /* reference count */
+	                                                 G_TYPE_BOOLEAN); /* connected */
 	navmodel->sorted = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (navmodel->store));
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (navmodel->sorted), 1, GTK_SORT_ASCENDING);
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (navmodel->sorted), 1, gtk_tree_iter_sort_func_nocase, NULL, NULL);
@@ -1299,7 +1339,15 @@ navigation_model_add_new_network (NavModel *model, struct session *sess)
 	gtk_tree_store_append (model->store, &iter, NULL);
 
 	/* Add the new server to the model. By default connected is false. */
-	gtk_tree_store_set (model->store, &iter, 1, _("<none>"), 2, sess, 3, 0, 4, NULL, 5, 0, 6, FALSE, -1);
+	gtk_tree_store_set (model->store, &iter,
+	                    COLUMN_ICON,      NULL,
+	                    COLUMN_NAME,      _("<none>"),
+	                    COLUMN_SESSION,   sess,
+	                    COLUMN_STATUS,    0,
+	                    COLUMN_COLOR,     NULL,
+	                    COLUMN_REFCOUNT,  0,
+	                    COLUMN_CONNECTED, FALSE,
+	                    -1);
 
 	/* Insert a row reference to the new entry in the unsorted store in the
 	 * hash table. This allows us to look up entries in the navigation model
@@ -1327,7 +1375,14 @@ navigation_model_create_new_channel_entry_iterate (GtkTreeModel *model, GtkTreeP
 
 		gtk_tree_store_append (GTK_TREE_STORE (model), &child, iter);
 		/* Add the new channel to the store. By default connected is true. */
-		gtk_tree_store_set (GTK_TREE_STORE (model), &child, 1, data->channel, 2, data, 3, 0, 4, NULL, 5, 0, 6, TRUE, -1);
+		gtk_tree_store_set (GTK_TREE_STORE (model), &child,
+		                    COLUMN_NAME,      data->channel,
+		                    COLUMN_SESSION,   data,
+		                    COLUMN_STATUS,    0,
+		                    COLUMN_COLOR,     NULL,
+		                    COLUMN_REFCOUNT,  0,
+		                    COLUMN_CONNECTED, TRUE,
+		                    -1);
 
 		path = gtk_tree_model_get_path (model, &child);
 		rowref = gtk_tree_row_reference_new (model, path);
@@ -1355,8 +1410,9 @@ navigation_model_remove (NavModel *model, struct session *sess)
 	GtkTreeIter *iter = navigation_model_get_unsorted_iter (model, sess);
 	GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (model->store), iter);
 
-	if (gtk_tree_path_get_depth (path) == 1)
+	if (gtk_tree_path_get_depth (path) == 1) {
 		model->servers--;
+	}
 
 	gtk_tree_store_remove (model->store, iter);
 
@@ -1406,7 +1462,10 @@ navigation_model_set_disconn_iterate (GtkTreeModel * model, GtkTreePath * path,
 	gtk_tree_model_get (model, iter, 2, &s, -1);
 	if (s == data) {
 		/* Change the color and set connected to false. */
-		gtk_tree_store_set (GTK_TREE_STORE (model), iter, 4, &colors[40], 6, FALSE, -1);
+		gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+		                    COLUMN_COLOR,     &colors[40],
+		                    COLUMN_CONNECTED, FALSE,
+		                    -1);
 		return TRUE;
 	}
 	return FALSE;
@@ -1430,18 +1489,31 @@ navigation_model_set_hilight_iterate (GtkTreeModel * model, GtkTreePath * path,
 		struct session *sess = s;
 
 		if (sess->nick_said) {
-			gtk_tree_store_set (GTK_TREE_STORE (model), iter, 0, pix_nicksaid, 3, 3, -1);
+			gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+			                    COLUMN_ICON, "xchat-gnome-message-nickname-said",
+			                    COLUMN_STATUS, 3,
+			                    -1);
 			return TRUE;
 		}
 		if (sess->msg_said && e < 2) {
-			if (sess->type != SESS_DIALOG)
-				gtk_tree_store_set (GTK_TREE_STORE (model), iter, 0, pix_msgsaid, 3, 2, -1);
-			else
-				gtk_tree_store_set (GTK_TREE_STORE (model), iter, 0, pix_nicksaid, 3, 3, -1);
+			if (sess->type != SESS_DIALOG) {
+				gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+				                    COLUMN_ICON, "xchat-gnome-message-new",
+				                    COLUMN_STATUS, 2,
+				                    -1);
+			} else {
+				gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+				                    COLUMN_ICON, "xchat-gnome-message-nickname-said",
+				                    COLUMN_STATUS, 3,
+				                    -1);
+			}
 			return TRUE;
 		}
 		if (sess->new_data && e < 1) {
-			gtk_tree_store_set (GTK_TREE_STORE (model), iter, 0, pix_newdata, 3, 1, -1);
+			gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+			                    COLUMN_ICON, "xchat-gnome-message-data",
+			                    COLUMN_STATUS, 1,
+			                    -1);
 			return TRUE;
 		}
 	}
@@ -1475,7 +1547,9 @@ navigation_model_rowref_ref (GtkTreeRowReference * rowref)
 	gtk_tree_model_get_iter (model, &iter, path);
 	gtk_tree_model_get (model, &iter, 5, &ref_count, -1);
 	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &childiter, &iter);
-	gtk_tree_store_set (GTK_TREE_STORE (childmodel), &childiter, 5, ref_count + 1, -1);
+	gtk_tree_store_set (GTK_TREE_STORE (childmodel), &childiter,
+	                    COLUMN_REFCOUNT, ref_count + 1,
+	                    -1);
 	gtk_tree_path_free (path);
 }
 
@@ -1490,6 +1564,10 @@ navigation_model_rowref_deref (GtkTreeRowReference *rowref)
 	model = gtk_tree_row_reference_get_model (rowref);
 	path = gtk_tree_row_reference_get_path (rowref);
 
+	if (NULL == path) {
+		return;
+	}
+
 	if (gtk_tree_model_get_iter (model, &iter, path) == FALSE) {
 		g_critical ("path is invalid in navigation_model_rowref_deref\n");
 		return;
@@ -1503,7 +1581,9 @@ navigation_model_rowref_deref (GtkTreeRowReference *rowref)
 
 		childmodel = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
 		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &childiter, &iter);
-		gtk_tree_store_set (GTK_TREE_STORE (childmodel), &childiter, 5, ref_count - 1, -1);
+		gtk_tree_store_set (GTK_TREE_STORE (childmodel), &childiter,
+		                    COLUMN_REFCOUNT, ref_count - 1,
+		                    -1);
 	}
 
 	gtk_tree_path_free (path);
@@ -1517,7 +1597,9 @@ navigation_model_sorted_iter_ref (NavModel * model, GtkTreeIter * iter)
 	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model->sorted), &unsorted, iter);
 	gtk_tree_model_get (model->sorted, iter, 5, &ref_count, -1);
 
-	gtk_tree_store_set (model->store, &unsorted, 5, ref_count + 1, -1);
+	gtk_tree_store_set (model->store, &unsorted,
+	                    COLUMN_REFCOUNT, ref_count + 1,
+	                    -1);
 }
 
 void
@@ -1529,8 +1611,11 @@ navigation_model_sorted_iter_unref (NavModel * model, GtkTreeIter * iter)
 	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model->sorted), &unsorted, iter);
 	gtk_tree_model_get (model->sorted, iter, 5, &ref_count, -1);
 
-	if (ref_count > 0)
-		gtk_tree_store_set (model->store, &unsorted, 5, ref_count - 1, -1);
+	if (ref_count > 0) {
+		gtk_tree_store_set (model->store, &unsorted,
+		                    COLUMN_REFCOUNT, ref_count - 1,
+		                    -1);
+	}
 }
 
 struct server *
@@ -1540,15 +1625,21 @@ navigation_model_get_server (NavModel *model, ircnet *network)
 	struct server *serv;
 	GtkTreeIter iter;
 
-	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model->store), &iter) == FALSE)
+	if (model == NULL || network == NULL) {
 		return NULL;
+	}
+
+	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model->store), &iter) == FALSE) {
+		return NULL;
+	}
 
 	do {
 		gtk_tree_model_get (GTK_TREE_MODEL (model->store), &iter, 2, &sess, -1);
 		if (sess) {
 			serv = sess->server;
-			if (serv->network == network)
+			if (serv && serv->network == network) {
 				return serv;
+			}
 		}
 	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (model->store), &iter));
 	return NULL;
@@ -1565,8 +1656,9 @@ on_server_reconnect (GtkAction * action, gpointer data)
 	session *s;
 
 	s = navigation_tree_get_selected_session (NULL);
-	if (s)
+	if (s) {
 		s->server->auto_reconnect (s->server, FALSE, -1);
+	}
 }
 
 static void
@@ -1582,8 +1674,9 @@ on_server_disconnect (GtkAction * action, gpointer data)
 
 	treeview = GTK_TREE_VIEW (gui.server_tree);
 	select = gtk_tree_view_get_selection (treeview);
-	if (!gtk_tree_selection_get_selected (select, &model, &iter))
+	if (!gtk_tree_selection_get_selected (select, &model, &iter)) {
 		return;
+	}
 
 	gtk_tree_model_get (model, &iter, 2, &s, -1);
 	if (s) {
@@ -1597,7 +1690,9 @@ on_server_disconnect (GtkAction * action, gpointer data)
 	/* Mark as disconnected */
 	store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
 	gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &newiter, &iter);
-	gtk_tree_store_set (GTK_TREE_STORE (store), &newiter, 6, FALSE, -1);
+	gtk_tree_store_set (GTK_TREE_STORE (store), &newiter,
+	                    COLUMN_CONNECTED, FALSE,
+	                    -1);
 }
 
 static void
@@ -1606,8 +1701,9 @@ on_server_channel_list (GtkAction * action, gpointer data)
 	session *s;
 
 	s = navigation_tree_get_selected_session (NULL);
-	if (s)
+	if (s) {
 		create_channel_list (s);
+	}
 }
 
 static void
@@ -1676,7 +1772,10 @@ on_channel_leave (GtkAction * action, gpointer data)
 			g_free (text);
 		}
 		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &newiter, &iter);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &newiter, 4, &colors[40], 6, FALSE, -1);
+		gtk_tree_store_set (GTK_TREE_STORE (store), &newiter,
+		                    COLUMN_COLOR,     &colors[40],
+		                    COLUMN_CONNECTED, FALSE,
+		                    -1);
 	}
 }
 
@@ -1772,9 +1871,9 @@ on_channel_autojoin (GtkAction *action, gpointer data)
 
 			for (i=0; channels[i]; i++) {
 				if (strcmp (channels[i], sess->channel) != 0) {
-					if (new_channels == NULL)
+					if (new_channels == NULL) {
 						new_channels = g_strdup (channels[i]);
-					else {
+					} else {
 						tmp = new_channels;
 						new_channels = g_strdup_printf ("%s,%s", new_channels, channels[i]);
 						g_free (tmp);
@@ -1788,21 +1887,21 @@ on_channel_autojoin (GtkAction *action, gpointer data)
 							new_keys = g_strdup_printf ("%s,%s", new_keys, keys[i]);
 							g_free (tmp);
 						}
-					}
-					else
+					} else {
 						keys_done = TRUE;
-				}
-				else if (!keys_done && !keys[i])
+					}
+				} else if (!keys_done && !keys[i]) {
 					keys_done = TRUE;
+				}
 			}
 
 			if (new_keys) {
 				network->autojoin = g_strdup_printf ("%s %s", new_channels, new_keys);
 				g_free (new_channels);
 				g_free (new_keys);
-			}
-			else
+			} else {
 				network->autojoin = new_channels;
+			}
 
 			servlist_save ();
 			g_strfreev (channels);
@@ -1814,23 +1913,25 @@ on_channel_autojoin (GtkAction *action, gpointer data)
 			/* FIXME: we should save the key of the channel is there is one */
 			if (network->autojoin == NULL) {
 				network->autojoin = g_strdup (sess->channel);
-			}
-			else {
+			} else {
 				autojoins = g_strsplit (network->autojoin, " ", 0);
 				tmp = network->autojoin;
 
-				if (autojoins[1])
+				if (autojoins[1]) {
 					network->autojoin = g_strdup_printf ("%s,%s %s", autojoins[0], sess->channel, autojoins[1]);
-				else
+				} else {
 					network->autojoin = g_strdup_printf ("%s,%s", autojoins[0], sess->channel);
+				}
 
 				g_free (tmp);
 			}
 
 			servlist_save ();
 		}
-		
-		if (autojoins) g_strfreev (autojoins);
+
+		if (autojoins) {
+			g_strfreev (autojoins);
+		}
 	}
 }
 
@@ -1845,9 +1946,11 @@ channel_is_autojoin (ircnet *network, gchar *chan)
 
 	autojoin = g_strsplit (network->autojoin, " ", 0);
 	channels = g_strsplit (autojoin[0], ",", 0);
-	for (i=0; channels[i] && !find; i++)
-		if (!strcmp (channels[i], chan))
+	for (i=0; channels[i] && !find; i++) {
+		if (!strcmp (channels[i], chan)) {
 			find = TRUE;
+		}
+	}
 
 	g_strfreev (autojoin);
 	g_strfreev (channels);
